/*------------------------------------------------------------------------/
/  Display control module include file
/-------------------------------------------------------------------------/
/
/  Copyright (C) 2014, ChaN, all right reserved.
/
/ * This software is a free software and there is NO WARRANTY.
/ * No restriction on use. You can use, modify and redistribute it for
/   personal, non-profit or commercial products UNDER YOUR RESPONSIBILITY.
/ * Redistributions of source code must retain the above copyright notice.
/
/-------------------------------------------------------------------------*/

#ifndef _DISPLAY
#define _DISPLAY
#include <stdint.h>

#define DISP_XS		128		/* Physical display width */
#define DISP_YS		128		/* Physical display height */

#define DISP_USE_DBCS	0	/* 1:Use DBCS string */


/* Physical display pixel data format */
#define RGB16(r,g,b)	(uint16_t)(((r) & 0xF8) << 8 | ((g) & 0xFC) << 3 | ((b) & 0xF8) >> 3)


/* Screen descriptor */
typedef struct {
	void* bitmap;			/* Pointer to the bitmap (null:physical device) */
	int width, height;		/* Size of bitmap */
	uint8_t bpp;			/* Color depth: 1, 4, 8 or 16 */
	uint8_t mode;			/* Operation mode (0:replace, 1:and, 2:or, 3:xor) */
	uint16_t color;			/* Drawing color */
	int x, y;				/* Current position */
	const uint16_t* palette;/* Ptr to pallete table (1/4/8 bpp only) */
	const uint8_t *font_s;	/* Ptr to SBC font (FONTX) */
#if DISP_USE_DBCS
	const uint8_t *font_d;	/* Ptr to DBC font (FONTX) */
#endif
} BMPDEV;


/* Display direct access interface */
void disp_power_on ();
void disp_power_off ();
void disp_setrect (int left, int top, int right, int bottom);
void disp_pixel (uint16_t pix, int n);
void disp_block_pixel (const uint16_t* pix, int n);


/* Virtual screen and drawing interface */
int disp_init (BMPDEV* bmd, int width, int height, uint8_t bpp);
void disp_box (BMPDEV* bmd, int x1, int y1, int x2, int y2);
void disp_fill (BMPDEV* bmd, int x1, int y1, int x2, int y2);
void disp_point (BMPDEV* bmd, int x, int y);
void disp_lineto (BMPDEV* bmd, int x, int y);
void disp_putc (BMPDEV* bmd, uint8_t chr);
void disp_transfer (BMPDEV* s_bmd, int s_left, int s_top, int s_right, int s_bottom, int d_left, int d_top);


#endif	/* _DISPLAY */
