/*------------------------------------------------------------------------/
/  LPC15xx USART basic control module
/-------------------------------------------------------------------------/
/
/  Copyright (C) 2014, ChaN, all right reserved.
/
/ * This software is a free software and there is NO WARRANTY.
/ * No restriction on use. You can use, modify and redistribute it for
/   personal, non-profit or commercial products UNDER YOUR RESPONSIBILITY.
/ * Redistributions of source code must retain the above copyright notice.
/
/-------------------------------------------------------------------------*/

#include "uart15xx.h"

static
void uartclk_init (void)
{
	/* Configure USART fractional divider */
	FRGCTRL = (uint32_t)((((double)F_MAINCLK / F_UARTCLK) / (F_MAINCLK / F_UARTCLK) - 1) * 256) << 8 | 0xFF;

	/* Configure USART main divider */
	UARTCLKDIV = F_MAINCLK / F_UARTCLK;
}


#if USE_UART0
#if UART0_TXB
static volatile struct {
	uint16_t	ri, wi, ct;
	uint8_t		buff[UART0_TXB];
} TxBuff0;
#endif
#if UART0_RXB
static volatile struct {
	uint16_t	ri, wi, ct;
	uint8_t		buff[UART0_RXB];
} RxBuff0;
#endif


#if UART0_TXB || UART0_RXB
void UART0_IRQHandler (void)
{
	uint16_t stat;
	uint8_t d;
	int i, cnt;


	stat = USART0_STAT;
#if UART0_RXB
	if (stat & 0x0001) {
		d = USART0_RXDAT;
		cnt = RxBuff0.ct;
		if (cnt < UART0_RXB) {	/* Store data if Rx buffer is not full */
			i = RxBuff0.wi;
			RxBuff0.buff[i++] = d;
			RxBuff0.wi = i % UART0_RXB;
			RxBuff0.ct = ++cnt;
		}
	}
#endif
#if UART0_TXB
	if (stat & 0x0004) {
		cnt = TxBuff0.ct;
		if (cnt) {		/* There is one or more byte to send */
			TxBuff0.ct = --cnt;
			i = TxBuff0.ri;
			d = TxBuff0.buff[i++];
			USART0_TXDAT = d;
			TxBuff0.ri = i % UART0_TXB;
		} else {
			USART0_INTENCLR = 0x0004;	/* Disable Tx interrupt */
		}
	}
#endif
}
#endif


int uart0_test (void)
{
#if UART0_RXB
	return RxBuff0.ct;
#else
	return (int)((USART0_STAT & 0x0001) != 0);
#endif
}


uint8_t uart0_getc (void)
{
	uint8_t d;

#if UART0_RXB
	/* Wait while Rx buffer is empty */
	while (!RxBuff0.ct) ;
	{
		int i;

		i = RxBuff0.ri;			/* Get a byte from Rx buffer */
		d = RxBuff0.buff[i++];
		RxBuff0.ri = i % UART0_RXB;
		__disable_irq();
		RxBuff0.ct--;
		__enable_irq();
	}
#else
	while (!(USART0_STAT & 0x0001)) ;
	d = USART0_RXDAT;
#endif

	return d;
}


void uart0_putc (uint8_t d)
{
#if UART0_TXB
	/* Wait for Tx buffer ready */
	while (TxBuff0.ct >= UART0_TXB) ;
	{
		int i;
		i = TxBuff0.wi;		/* Put a byte into Tx byffer */
		TxBuff0.buff[i++] = d;
		TxBuff0.wi = i % UART0_TXB;
		__disable_irq();
		TxBuff0.ct++;
		USART0_INTENSET = 0x0004;
		__enable_irq();
	}
#else
	while (!(USART0_STAT & 0x0004)) ;
	USART0_TXDAT = d;
#endif
}

void uart0_init (uint32_t bps)
{
	__disable_irqn(UART0_IRQn);
	uartclk_init();

	/* Enable UART0 module */
	__enable_ahbclk(PCUART0);

	/* Initialize UART */
	USART0_BRG = F_UARTCLK / ((bps ? bps : UART0_BPS) * 16) - 1;	/* Bit rate */
	USART0_CFG = 0x0005;	/* Format: N81 */

	/* Clear Tx/Rx buffers */
#if UART0_TXB
	TxBuff0.ri = 0; TxBuff0.wi = 0; TxBuff0.ct = 0;
#endif
#if UART0_RXB
	RxBuff0.ri = 0; RxBuff0.wi = 0; RxBuff0.ct = 0;
	USART0_INTENSET = 0x0001;
#endif
#if UART0_TXB || UART0_RXB
	__enable_irqn(UART0_IRQn);
#endif
}

#endif	/* USE_UART0 */


#if USE_UART1

#if UART1_TXB
static volatile struct {
	uint16_t	ri, wi, ct;
	uint8_t		buff[UART1_TXB];
} TxBuff1;
#endif
#if UART1_RXB
static volatile struct {
	uint16_t	ri, wi, ct;
	uint8_t		buff[UART1_RXB];
} RxBuff1;
#endif


#if UART1_TXB || UART1_RXB
void UART1_IRQHandler (void)
{
	uint16_t stat;
	uint8_t d;
	int i, cnt;


	stat = USART1_STAT;
#if UART1_RXB
	if (stat & 0x0001) {
		d = USART1_RXDAT;
		cnt = RxBuff1.ct;
		if (cnt < UART1_RXB) {	/* Store data if Rx buffer is not full */
			i = RxBuff1.wi;
			RxBuff1.buff[i++] = d;
			RxBuff1.wi = i % UART1_RXB;
			RxBuff1.ct = ++cnt;
		}
	}
#endif
#if UART1_TXB
	if (stat & 0x0004) {
		cnt = TxBuff1.ct;
		if (cnt) {		/* There is one or more byte to send */
			TxBuff1.ct = --cnt;
			i = TxBuff1.ri;
			d = TxBuff1.buff[i++];
			USART1_TXDAT = d;
			TxBuff1.ri = i % UART1_TXB;
		} else {
			USART1_INTENCLR = 0x0004;	/* Disable Tx interrupt */
		}
	}
#endif
}
#endif


int uart1_test (void)
{
#if UART1_RXB
	return RxBuff1.ct;
#else
	return (int)((USART1_STAT & 0x0001) != 0);
#endif
}


uint8_t uart1_getc (void)
{
	uint8_t d;

#if UART1_RXB
	/* Wait while Rx buffer is empty */
	while (!RxBuff1.ct) ;
	{
		int i;

		i = RxBuff1.ri;			/* Get a byte from Rx buffer */
		d = RxBuff1.buff[i++];
		RxBuff1.ri = i % UART1_RXB;
		__disable_irq();
		RxBuff1.ct--;
		__enable_irq();
	}
#else
	while (!(USART1_STAT & 0x0001)) ;
	d = USART1_RXDAT;
#endif

	return d;
}


void uart1_putc (uint8_t d)
{
#if UART1_TXB
	/* Wait for Tx buffer ready */
	while (TxBuff1.ct >= UART1_TXB) ;
	{
		int i;
		i = TxBuff1.wi;		/* Put a byte into Tx byffer */
		TxBuff1.buff[i++] = d;
		TxBuff1.wi = i % UART1_TXB;
		__disable_irq();
		TxBuff1.ct++;
		USART1_INTENSET = 0x0004;
		__enable_irq();
	}
#else
	while (!(USART1_STAT & 0x0004)) ;
	USART1_TXDAT = d;
#endif
}


void uart1_init (uint32_t bps)
{
	__disable_irqn(UART1_IRQn);
	uartclk_init();

	/* Enable UART1 module */
	__enable_ahbclk(PCUART1);

	/* Initialize UART */
	USART1_BRG = F_UARTCLK / ((bps ? bps : UART1_BPS) * 16) - 1;	/* Bit rate */
	USART1_CFG = 0x0005;	/* Format: N81 */

	/* Clear Tx/Rx buffers */
#if UART1_TXB
	TxBuff1.ri = 0; TxBuff1.wi = 0; TxBuff1.ct = 0;
#endif
#if UART1_RXB
	RxBuff1.ri = 0; RxBuff1.wi = 0; RxBuff1.ct = 0;
	USART1_INTENSET = 0x0001;
#endif
#if UART1_TXB || UART1_RXB
	__enable_irqn(UART1_IRQn);
#endif
}

#endif	/* USE_UART1 */


#if USE_UART2

#if UART2_TXB
static volatile struct {
	uint16_t	ri, wi, ct;
	uint8_t		buff[UART2_TXB];
} TxBuff2;
#endif
#if UART2_RXB
static volatile struct {
	uint16_t	ri, wi, ct;
	uint8_t		buff[UART2_RXB];
} RxBuff2;
#endif


#if UART2_TXB || UART2_RXB
void UART2_IRQHandler (void)
{
	uint16_t stat;
	uint8_t d;
	int i, cnt;


	stat = USART2_STAT;
#if UART2_RXB
	if (stat & 0x0001) {
		d = USART2_RXDAT;
		cnt = RxBuff2.ct;
		if (cnt < UART2_RXB) {	/* Store data if Rx buffer is not full */
			i = RxBuff2.wi;
			RxBuff2.buff[i++] = d;
			RxBuff2.wi = i % UART2_RXB;
			RxBuff2.ct = ++cnt;
		}
	}
#endif
#if UART2_TXB
	if (stat & 0x0004) {
		cnt = TxBuff2.ct;
		if (cnt) {		/* There is one or more byte to send */
			TxBuff2.ct = --cnt;
			i = TxBuff2.ri;
			d = TxBuff2.buff[i++];
			USART2_TXDAT = d;
			TxBuff2.ri = i % UART2_TXB;
		} else {
			USART2_INTENCLR = 0x0004;	/* Disable Tx interrupt */
		}
	}
#endif
}
#endif


int uart2_test (void)
{
#if UART2_RXB
	return RxBuff2.ct;
#else
	return (int)((USART2_STAT & 0x0001) != 0);
#endif
}


uint8_t uart2_getc (void)
{
	uint8_t d;

#if UART2_RXB
	/* Wait while Rx buffer is empty */
	while (!RxBuff2.ct) ;
	{
		int i;

		i = RxBuff2.ri;			/* Get a byte from Rx buffer */
		d = RxBuff2.buff[i++];
		RxBuff2.ri = i % UART2_RXB;
		__disable_irq();
		RxBuff2.ct--;
		__enable_irq();
	}
#else
	while (!(USART2_STAT & 0x0001)) ;
	d = USART2_RXDAT;
#endif

	return d;
}


void uart2_putc (uint8_t d)
{
#if UART2_TXB
	/* Wait for Tx buffer ready */
	while (TxBuff2.ct >= UART2_TXB) ;
	{
		int i;
		i = TxBuff2.wi;		/* Put a byte into Tx byffer */
		TxBuff2.buff[i++] = d;
		TxBuff2.wi = i % UART2_TXB;
		__disable_irq();
		TxBuff2.ct++;
		USART2_INTENSET = 0x0004;
		__enable_irq();
	}
#else
	while (!(USART2_STAT & 0x0004)) ;
	USART2_TXDAT = d;
#endif
}


void uart2_init (uint32_t bps)
{
	__disable_irqn(UART2_IRQn);
	uartclk_init();

	/* Enable UART2 module */
	__enable_ahbclk(PCUART2);

	/* Initialize UART */
	USART2_BRG = F_UARTCLK / ((bps ? bps : UART2_BPS) * 16) - 1;	/* Bit rate */
	USART2_CFG = 0x0005;	/* Format: N81 */

	/* Clear Tx/Rx buffers */
#if UART2_TXB
	TxBuff2.ri = 0; TxBuff2.wi = 0; TxBuff2.ct = 0;
#endif
#if UART2_RXB
	RxBuff2.ri = 0; RxBuff2.wi = 0; RxBuff2.ct = 0;
	USART2_INTENSET = 0x0001;
#endif
#if UART2_TXB || UART2_RXB
	__enable_irqn(UART2_IRQn);
#endif
}

#endif	/* USE_UART2 */

